@@ -331,7 +331,33 @@ class TMC_2209:
 #-----------------------------------------------------------------------
 # homes the motor in the given direction using stallguard
 #-----------------------------------------------------------------------
-    def doHoming(self, direction, threshold=None):
+    def doHoming(self, direction, diag_pin, threshold=None):        
+        if(threshold is not None):
+            self._sg_threshold = threshold
+        
+        self.log("---", Loglevel.info.value)
+        self.log("homing", Loglevel.info.value)
+
+        self.setSpreadCycle(0)
+
+        self.log("Stallguard threshold:"+str(self._sg_threshold), Loglevel.debug.value)
+
+        self.setStallguard_Callback(diag_pin, self._sg_threshold, self.stop)
+        
+        if(direction):
+            self.setVActual_rpm(30, revolutions=1)
+        else:
+            self.setVActual_rpm(30, revolutions=-1)
+        self._currentPos = 0
+        
+        self.log("---", Loglevel.info.value)
+        
+
+#-----------------------------------------------------------------------
+# homes the motor in the given direction using stallguard
+# old function, uses STEP/DIR 
+#-----------------------------------------------------------------------
+    def doHoming2(self, direction, threshold=None):
         sg_results = []
         
         if(threshold is not None):
@@ -372,7 +398,6 @@ class TMC_2209:
 
         if(step_counter<self._stepsPerRevolution):
             self.log("homing successful",Loglevel.info.value)
-            self.log("Stepcounter: "+str(step_counter),Loglevel.info.value)
             self.log("Stepcounter: "+str(step_counter),Loglevel.debug.value)
             self.log(str(sg_results),Loglevel.debug.value)
             self._currentPos = 0
@@ -431,7 +456,7 @@ class TMC_2209:
             self.log("write inverse motor direction", Loglevel.info.value)
             gconf = self.tmc_uart.set_bit(gconf, reg.shaft)
         else:
-            self.log("write normal motor direction")
+            self.log("write normal motor direction", Loglevel.info.value)
             gconf = self.tmc_uart.clear_bit(gconf, reg.shaft)
         self.tmc_uart.write_reg_check(reg.GCONF, gconf)
   
@@ -503,6 +528,9 @@ class TMC_2209:
         gconf = self.tmc_uart.read_int(reg.GCONF)
         if(en):
             self.log("activated internal sense resistors.", Loglevel.info.value)
+            self.log("VREF pin internally is driven to GND in this mode.", Loglevel.info.value)
+            self.log("This will most likely destroy your driver!!!", Loglevel.info.value)
+            raise SystemExit
             gconf = self.tmc_uart.set_bit(gconf, reg.internal_rsense)
         else:
             self.log("activated operation with external sense resistors", Loglevel.info.value)
@@ -721,13 +749,47 @@ class TMC_2209:
 # It gives the motor velocity in +-(2^23)-1 [μsteps / t]
 # 0: Normal operation. Driver reacts to STEP input
 #-----------------------------------------------------------------------
-    def setVActual(self, vactual):
-        self.log("vactual", Loglevel.info.value)
+    def setVActual(self, vactual, duration=0):
+        self._stop = False
+
+        if(duration != 0):
+            self.log("vactual: "+str(vactual)+" for "+str(duration)+" sec", Loglevel.info.value)
+        else:
+            self.log("vactual: "+str(vactual), Loglevel.info.value)
         self.log(str(bin(vactual)), Loglevel.info.value)
 
         self.log("writing vactual", Loglevel.info.value)
         self.tmc_uart.write_reg_check(reg.VACTUAL, vactual)
 
+        if(duration != 0):
+            starttime = time.time()
+            while((not self._stop) and (time.time() < starttime+duration)):
+                pass
+            self.tmc_uart.write_reg_check(reg.VACTUAL, 0)
+            return not self._stop
+
+
+
+#-----------------------------------------------------------------------
+# converts the rps parameter to a vactual value which represents
+# rotation speed in revolutions per second
+# With internal oscillator:
+# VACTUAL[2209] = v[Hz] / 0.715Hz
+#-----------------------------------------------------------------------
+    def setVActual_rps(self, rps, duration=0, revolutions=0):
+        vactual = rps/0.715*self._stepsPerRevolution
+        if(revolutions!=0):
+            duration = abs(revolutions/rps)
+        return self.setVActual(int(round(vactual)), duration)
+
+
+#-----------------------------------------------------------------------
+# converts the rps parameter to a vactual value which represents
+# rotation speed in revolutions per minute
+#-----------------------------------------------------------------------
+    def setVActual_rpm(self, rpm, duration=0, revolutions=0):
+        return self.setVActual_rps(rpm/60, duration, revolutions)
+
 
 #-----------------------------------------------------------------------
 # return the current stallguard result
